package com.robotoworks.mechanoid.net.generator

import com.google.inject.Inject
import com.robotoworks.mechanoid.net.netModel.Client
import com.robotoworks.mechanoid.net.netModel.HttpMethod
import com.robotoworks.mechanoid.net.netModel.HttpMethodType
import com.robotoworks.mechanoid.net.netModel.Model

import static extension com.robotoworks.mechanoid.net.generator.ModelExtensions.*
import static extension com.robotoworks.mechanoid.text.Strings.*

class ClientGenerator {
	@Inject ImportHelper imports
	
	def generate(Client client, Model module) '''
	package module.packageName;

	var classDecl = generateClientClass(client, module)
	
	import com.robotoworks.mechanoid.net.Parser;
	import java.io.IOException;
	import com.robotoworks.mechanoid.net.Response;
	import com.robotoworks.mechanoid.net.ServiceException;
	import java.io.InputStream;
	import com.robotoworks.mechanoid.net.ServiceClient;
	import com.robotoworks.mechanoid.net.JsonEntityWriterProvider;
	import com.robotoworks.mechanoid.net.JsonEntityReaderProvider;
	imports.printAndClear
	
	classDecl
	'''
	
	def generateClientClass(Client client, Model module) '''
		public abstract class Abstractclient.name extends ServiceClient {
			private static final String LOG_TAG = "client.name";
			
			IF client.baseUrl != null
			protected static final String DEFAULT_BASE_URL = "client.baseUrl";
			
			ENDIF
			var params = client.paramsBlock
			IF(params != null)
			FOR param:params.params
			private param.member.type.signature param.member.name.camelizeParamIF param.defaultValue != null = param.defaultValue.convertToJavaLiteralENDIF;
			private boolean param.member.name.camelizeParamSetIF param.defaultValue != null = trueENDIF;
			ENDFOR
			
			ENDIF
			IF(params != null)
			FOR param:params.params
			public void setparam.member.name.pascalizeParam(param.member.type.signature value) {
				this.param.member.name.camelizeParam = value;
				this.param.member.name.camelizeParamSet = true;
			}
			ENDFOR
			
			ENDIF
			@Override
			protected String getLogTag() {
				return LOG_TAG;
			}
			
			@Override
			protected JsonEntityWriterProvider createWriterProvider() {
				return new Defaultclient.nameWriterProvider();
			}
			
			@Override
			protected JsonEntityReaderProvider createReaderProvider() {
				return new Defaultclient.nameReaderProvider();
			}
			
			public Abstractclient.name(String baseUrl, boolean debug){
				super(baseUrl, debug);
				
				var headers = client.headerBlock
				IF headers != null
				FOR header : headers.headers
				setHeader("header.name","header.value");
				ENDFOR
				ENDIF
			}
			
			generateClientMethods(client, module)
		}
	'''
	
	def generateClientMethods(Client client, Model model) '''
		FOR method:client.blocks.filter(typeof(HttpMethod))
		IF !method.hasBody && (method.path == null || method.path.params.size == 0)
		public Response<method.name.pascalizeResult> method.name.camelize()
		  throws ServiceException {
		  	return method.name.camelize(new method.name.pascalizeRequest());
		}
		
		ENDIF
		public Response<method.name.pascalizeResult> method.name.camelize(method.name.pascalizeRequest request)
		  throws ServiceException {
		  	var params = client.paramsBlock
			IF(params != null)
			FOR param:params.params
			if(this.param.member.name.camelizeParamSet && !request.isparam.member.name.pascalizeParamSet()){
				request.setparam.member.name.pascalizeParam(this.param.member.name.camelizeParam);
			}
			ENDFOR
			
			ENDIF	
			
			Parser<method.name.pascalizeResult> parser = new Parser<method.name.pascalizeResult>() {
				public method.name.pascalizeResult parse(InputStream inStream) throws IOException {
					return new method.name.pascalizeResult(getReaderProvider(), inStream);
				}
			};
			
			generateServiceMethod(method)
		}
		
		ENDFOR
	'''
	def generateServiceMethod(HttpMethod method) { 
		switch(method.type) {
			case HttpMethodType::GET:
				generateServiceGetMethod(method)
			case HttpMethodType::PUT:
				generateServicePutMethod(method)
			case HttpMethodType::POST:
				generateServicePostMethod(method)
			case HttpMethodType::DELETE:
				generateServiceDeleteMethod(method)
				
		}
	}

	
	def generateServiceGetMethod(HttpMethod method) '''
		return get(request, parser);
	'''
	def printSetHeadersStatements() '''
		for(String key : headers.keySet()) {
			conn.setRequestProperty(key, headers.get(key));
		}
		
		for(String key : request.getHeaderKeys()) {
			conn.setRequestProperty(key, request.getHeaderValue(key));
		}
	'''

	def generateServicePutMethod(HttpMethod method) '''
		return put(request, parser);
	'''
	
	def generateServicePostMethod(HttpMethod method) '''
		return post(request, parser);
	'''
	def generateServiceDeleteMethod(HttpMethod method) '''
		return delete(request, parser);
	'''

	def generateStub(Client client, Model module) '''
	package module.packageName;
	
	public class client.name extends Abstractclient.name {
		IF client.baseUrl != null
		public client.name(){
			super(DEFAULT_BASE_URL, false);
		}

		public client.name(boolean debug){
			super(DEFAULT_BASE_URL, debug);
		}
		ENDIF
		
		public client.name(String baseUrl){
			super(baseUrl, false);
		}
		
		public client.name(String baseUrl, boolean debug){
			super(baseUrl, debug);
		}
	}
	'''
}